@@ -1,6 +1,6 @@
 /*
  *******************************************************************************
- * Copyright (C) 1996-2005, International Business Machines Corporation and    *
+ * Copyright (C) 1996-2008, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
@@ -217,7 +217,7 @@ public class CalendarTest extends TestFmwk {
      * @param testDuration if positive, the number of days to be tested.
      * If negative, the number of seconds to run the test.
      */
-    protected void doLimitsTest(Calendar cal, int[] fieldsToTest,
+    public void doLimitsTest(Calendar cal, int[] fieldsToTest,
                                 Date startDate, int testDuration) {
         GregorianCalendar greg = new GregorianCalendar();
         greg.setTime(startDate);
@@ -274,19 +274,19 @@ public class CalendarTest extends TestFmwk {
 
                 if (minActual < minLow || minActual > minHigh) {
                     errln("Fail: " + ymdToString(cal) +
-                          " Range for min of " + FIELD_NAME[f] +
-                          "=" + minLow + ".." + minHigh +
+                          " Range for min of " + FIELD_NAME[f] + "(" + f +
+                          ")=" + minLow + ".." + minHigh +
                           ", actual_min=" + minActual);
                 }
                 if (maxActual < maxLow || maxActual > maxHigh) {
                     errln("Fail: " + ymdToString(cal) +
-                          " Range for max of " + FIELD_NAME[f] +
-                          "=" + maxLow + ".." + maxHigh +
+                          " Range for max of " + FIELD_NAME[f] + "(" + f +
+                          ")=" + maxLow + ".." + maxHigh +
                           ", actual_max=" + maxActual);
                 }
                 if (v < minActual || v > maxActual) {
                     errln("Fail: " + ymdToString(cal) +
-                          " " + FIELD_NAME[f] + "=" + v +
+                          " " + FIELD_NAME[f] + "(" + f + ")=" + v +
                           ", actual range=" + minActual + ".." + maxActual +
                           ", allowed=(" + minLow + ".." + minHigh + ")..(" +
                           maxLow + ".." + maxHigh + ")");
@@ -335,6 +335,80 @@ public class CalendarTest extends TestFmwk {
         logln("End: " + greg.getTime());
     }
 
+    /**
+     * doLimitsTest with default test duration
+     */
+    public void doLimitsTest(Calendar cal, int[] fieldsToTest, Date startDate) {
+        int testTime = getInclusion() <= 5 ? -3 : -120; // in seconds
+        doLimitsTest(cal, fieldsToTest, startDate, testTime);
+    }
+    
+    /**
+     * Test the functions getMaximum/getGeratestMinimum logically correct.
+     * This method assumes day of week cycle is consistent.
+     * @param cal The calendar instance to be tested.
+     * @param leapMonth true if the calendar system has leap months
+     */
+    public void doTheoreticalLimitsTest(Calendar cal, boolean leapMonth) {
+        int nDOW = cal.getMaximum(Calendar.DAY_OF_WEEK);
+        int maxDOY = cal.getMaximum(Calendar.DAY_OF_YEAR);
+        int lmaxDOW = cal.getLeastMaximum(Calendar.DAY_OF_YEAR);
+        int maxWOY = cal.getMaximum(Calendar.WEEK_OF_YEAR);
+        int lmaxWOY = cal.getLeastMaximum(Calendar.WEEK_OF_YEAR);
+        int maxM = cal.getMaximum(Calendar.MONTH) + 1;
+        int lmaxM = cal.getLeastMaximum(Calendar.MONTH) + 1;
+        int maxDOM = cal.getMaximum(Calendar.DAY_OF_MONTH);
+        int lmaxDOM = cal.getLeastMaximum(Calendar.DAY_OF_MONTH);
+        int maxDOWIM = cal.getMaximum(Calendar.DAY_OF_WEEK_IN_MONTH);
+        int lmaxDOWIM = cal.getLeastMaximum(Calendar.DAY_OF_WEEK_IN_MONTH);
+        int maxWOM = cal.getMaximum(Calendar.WEEK_OF_MONTH);
+        int lmaxWOM = cal.getLeastMaximum(Calendar.WEEK_OF_MONTH);
+        int minDaysInFirstWeek = cal.getMinimalDaysInFirstWeek();
+
+        // Day of year
+        int expected;
+        if (!leapMonth) {
+            expected = maxM*maxDOM;
+            if (maxDOY > expected) {
+                errln("FAIL: Maximum value of DAY_OF_YEAR is too big: " + maxDOY + "/expected: <=" + expected);
+            }
+            expected = lmaxM*lmaxDOM;
+            if (lmaxDOW < expected) {
+                errln("FAIL: Least maximum value of DAY_OF_YEAR is too small: " + lmaxDOW + "/expected: >=" + expected);
+            }
+        }
+
+        // Week of year
+        expected = maxDOY/nDOW + 1;
+        if (maxWOY > expected) {
+            errln("FAIL: Maximum value of WEEK_OF_YEAR is too big: " + maxWOY + "/expected: <=" + expected);
+        }
+        expected = lmaxDOW/nDOW;
+        if (lmaxWOY < expected) {
+            errln("FAIL: Least maximum value of WEEK_OF_YEAR is too small: " + lmaxWOY + "/expected >=" + expected);
+        }
+
+        // Day of week in month
+        expected = (maxDOM + nDOW - 1)/nDOW;
+        if (maxDOWIM != expected) {
+            errln("FAIL: Maximum value of DAY_OF_WEEK_IN_MONTH is incorrect: " + maxDOWIM + "/expected: " + expected);
+        }
+        expected = (lmaxDOM + nDOW - 1)/nDOW;
+        if (lmaxDOWIM != expected) {
+            errln("FAIL: Least maximum value of DAY_OF_WEEK_IN_MONTH is incorrect: " + lmaxDOWIM + "/expected: " + expected);
+        }
+
+        // Week of month
+        expected = (maxDOM + (nDOW - 1) + (nDOW - minDaysInFirstWeek)) / nDOW;
+        if (maxWOM != expected) {
+            errln("FAIL: Maximum value of WEEK_OF_MONTH is incorrect: " + maxWOM + "/expected: " + expected);
+        }
+        expected = (lmaxDOM + (nDOW - minDaysInFirstWeek)) / nDOW;
+        if (lmaxWOM != expected) {
+            errln("FAIL: Least maximum value of WEEK_OF_MONTH is incorrect: " + lmaxWOM + "/expected: " + expected);
+        }
+    }
+    
     /**
      * Convert year,month,day values to the form "year/month/day".
      * On input the month value is zero-based, but in the result string it is one-based.
@@ -352,11 +426,11 @@ public class CalendarTest extends TestFmwk {
             return "" + cal.get(Calendar.EXTENDED_YEAR) + "/" +
                 (cal.get(Calendar.MONTH)+1) +
                 (cal.get(ChineseCalendar.IS_LEAP_MONTH)==1?"(leap)":"") + "/" +
-                cal.get(Calendar.DATE) + " (" + day + ")";
+                cal.get(Calendar.DATE) + " (" + day + ", time=" + cal.getTimeInMillis() + ")";
         }
         return ymdToString(cal.get(Calendar.EXTENDED_YEAR),
                             cal.get(MONTH), cal.get(DATE)) +
-                            " (" + day + ")";
+                            " (" + day + ", time=" + cal.getTimeInMillis() + ")";
     }
 
     static double getJulianDay(Calendar cal) {
