@@ -1,6 +1,6 @@
 /**
 *******************************************************************************
-* Copyright (C) 2006, International Business Machines Corporation and    *
+* Copyright (C) 2009, International Business Machines Corporation and    *
 * others. All Rights Reserved.                                                *
 *******************************************************************************
 *
@@ -23,7 +23,8 @@ public class TestAll extends TestGroup {
         super(
               new String[] {
                   "TestCharset",
-                  "TestConversion"
+                  "TestConversion",
+                  "TestSelection"
               },
               "All ICU Charset Tests"
               );
