@@ -1,6 +1,6 @@
 /*****************************************************************************************
  *
- *   Copyright (C) 1996-2006, International Business Machines
+ *   Copyright (C) 1996-2009, International Business Machines
  *   Corporation and others.  All Rights Reserved.
  **/
 
@@ -144,6 +144,26 @@ public class IntlTestDecimalFormatSymbols extends com.ibm.icu.dev.test.TestFmwk
         if(fr.getExponentSeparator() != en.getExponentSeparator()) {
             errln("ERROR: get/set Exponential failed");
         }
+        
+        // Test CurrencySpacing.
+        // In CLDR 1.7, only root.txt has CurrencySpacing data. This data might 
+        // be different between en and fr in the future.
+        for (int i = DecimalFormatSymbols.CURRENCY_SPC_CURRENCY_MATCH; i <= DecimalFormatSymbols.CURRENCY_SPC_INSERT; i++) {
+            if (en.getPatternForCurrencySpacing(i, true) !=
+                fr.getPatternForCurrencySpacing(i, true)) {
+                errln("ERROR: get currency spacing item:"+ i+" before the currency");
+                if (en.getPatternForCurrencySpacing(i, false) !=
+                    fr.getPatternForCurrencySpacing(i, false)) {
+                    errln("ERROR: get currency spacing item:" + i + " after currency");
+                }
+            }
+        }
+        
+        String dash = "-";
+        en.setPatternForCurrencySpacing(DecimalFormatSymbols.CURRENCY_SPC_INSERT, true, dash);
+        if (dash != en.getPatternForCurrencySpacing(DecimalFormatSymbols.CURRENCY_SPC_INSERT, true)) {
+            errln("ERROR: set currency spacing pattern for before currency.");
+        }
 
         //DecimalFormatSymbols foo = new DecimalFormatSymbols(); //The variable is never used
 
