@@ -1,7 +1,7 @@
-//##header J2SE15
+//##header
 /*****************************************************************************************
  *
- * Copyright (C) 1996-2007, International Business Machines
+ * Copyright (C) 1996-2009, International Business Machines
  * Corporation and others.  All Rights Reserved.
  **/
 
@@ -464,11 +464,11 @@ public class NumberRegression extends com.ibm.icu.dev.test.TestFmwk {
         String tempString;
         /* User error :
         String expectedDefault = "-5\u00a0789,987";
-        String expectedCurrency = "5\u00a0789,98 F";
+        String expectedCurrency = "5\u00a0789,98\u00a0F";
         String expectedPercent = "-578\u00a0998%";
         */
         String expectedDefault = "-5\u00a0789,988";
-        String expectedCurrency = "5\u00a0789,99 " + EURO; // euro
+        String expectedCurrency = "5\u00a0789,99\u00a0" + EURO; // euro
         String expectedPercent = "-578\u00a0999\u00a0%";
 
         formatter = NumberFormat.getNumberInstance(Locale.FRANCE);
@@ -515,11 +515,11 @@ public class NumberRegression extends com.ibm.icu.dev.test.TestFmwk {
         String tempString;
     /* user error :
         String expectedDefault = "-5 789,987";
-        String expectedCurrency = "5 789,98 $";
+        String expectedCurrency = "5 789,98\u00a0$";
         String expectedPercent = "-578 998%";
     */
         String expectedDefault = "-5\u00a0789,988";
-        String expectedCurrency = "5\u00a0789,99 $";
+        String expectedCurrency = "5\u00a0789,99\u00a0$";
         String expectedPercent = "-578\u00a0999\u00A0%";
 
         formatter = NumberFormat.getNumberInstance(Locale.CANADA_FRENCH);
@@ -562,12 +562,12 @@ public class NumberRegression extends com.ibm.icu.dev.test.TestFmwk {
         String tempString;
         /* user error :
         String expectedDefault = "-5.789,987";
-        String expectedCurrency = "5.789,98 DM";
+        String expectedCurrency = "5.789,98\u00a0DM";
         String expectedPercent = "-578.998%";
         */
         String expectedDefault = "-5.789,988";
-        String expectedCurrency = "5.789,99 " + EURO;
-        String expectedPercent = "-578.999 %";
+        String expectedCurrency = "5.789,99\u00a0" + EURO;
+        String expectedPercent = "-578.999\u00a0%";
 
         formatter = NumberFormat.getNumberInstance(Locale.GERMANY);
         tempString = formatter.format (-5789.9876);
@@ -612,11 +612,11 @@ public class NumberRegression extends com.ibm.icu.dev.test.TestFmwk {
         String tempString;
         /* user error :
         String expectedDefault = "-5.789,987";
-        String expectedCurrency = "-L. 5.789,98";
+        String expectedCurrency = "-L.\u00a05.789,98";
         String expectedPercent = "-578.998%";
         */
         String expectedDefault = "-5.789,988";
-        String expectedCurrency = "-" + EURO + " 5.789,99";
+        String expectedCurrency = "-" + EURO + "\u00a05.789,99";
         String expectedPercent = "-578.999%";
 
         formatter = NumberFormat.getNumberInstance(Locale.ITALY);
@@ -1107,7 +1107,7 @@ public class NumberRegression extends com.ibm.icu.dev.test.TestFmwk {
         }
     }
 
-//#if defined(FOUNDATION10) || defined(J2SE13)
+//#if defined(FOUNDATION10)
 //#else
     /**
      * BigDecimal numbers get their fractions truncated by NumberFormat.
@@ -1402,9 +1402,9 @@ public class NumberRegression extends com.ibm.icu.dev.test.TestFmwk {
             ois.close();
             
             if (!nf.equals(o)) {
-                errln("Fail: DateFormat serialization/equality bug");
+                errln("Fail: NumberFormat serialization/equality bug");
             } else {
-                logln("DateFormat serialization/equality is OKAY.");
+                logln("NumberFormat serialization/equality is OKAY.");
             }
         }
 
@@ -1845,7 +1845,7 @@ class MyNumberFormatTest extends NumberFormat {
     public Number parse(String text, ParsePosition parsePosition) {
         return new Integer(0);
     }
-//#if defined(FOUNDATION10) || defined(J2SE13)
+//#if defined(FOUNDATION10)
 //#else
     public StringBuffer format(java.math.BigDecimal number, StringBuffer toAppendTo, FieldPosition pos) {
         return new StringBuffer("");
