@@ -1,7 +1,7 @@
-//##header J2SE15
+//##header
 /*
 **********************************************************************
-* Copyright (c) 2004-2008, International Business Machines
+* Copyright (c) 2004-2009, International Business Machines
 * Corporation and others.  All Rights Reserved.
 **********************************************************************
 * Author: Alan Liu
@@ -27,9 +27,12 @@ import java.util.Set;
 
 import com.ibm.icu.text.DateFormat;
 import com.ibm.icu.text.DecimalFormat;
+import com.ibm.icu.text.DecimalFormatSymbols;
 import com.ibm.icu.text.MessageFormat;
 import com.ibm.icu.text.NumberFormat;
+import com.ibm.icu.text.SimpleDateFormat;
 import com.ibm.icu.text.UFormat;
+import com.ibm.icu.util.TimeZone;
 import com.ibm.icu.util.ULocale;
 
 public class TestMessageFormat extends com.ibm.icu.dev.test.TestFmwk {
@@ -1293,6 +1296,25 @@ public class TestMessageFormat extends com.ibm.icu.dev.test.TestFmwk {
         }
     }
 
+  // Test toPattern when there is a PluralFormat
+  public void testPluralFormatToPattern() {
+    String[] patterns = {
+      "Beware of vicious {0, plural, one {hamster} other {hamsters}}.",
+      "{0, plural, one {{0, number,C''''est #,##0.0# fichier}} other {Ce sont # fichiers}} dans la liste.",
+      "{0, plural, one {C''est # fichier} other {Ce sont # fichiers}} dans la liste.",
+    };
+
+    for (int i = 0; i < patterns.length; ++i) {
+      String pattern = patterns[i];
+      MessageFormat mf = new MessageFormat(pattern);
+      MessageFormat mf2 = new MessageFormat(mf.toPattern());
+      if (!mf.equals(mf2)) {
+        errln("message formats not equal for pattern:\n*** '" + pattern + "'\n*** '" +
+              mf.toPattern() + "'");
+      }
+    }
+  }
+
     // Test case for null arguments.
     // Ticket#6361
     public void TestNullArgs() {
@@ -1313,6 +1335,24 @@ public class TestMessageFormat extends com.ibm.icu.dev.test.TestFmwk {
             }
         }
     }
+    
+    public void TestSetFormat() {
+        MessageFormat ms = new MessageFormat("{number} {date}", ULocale.ENGLISH);
+        final DecimalFormat decimalFormat = new DecimalFormat("000.000", DecimalFormatSymbols.getInstance(ULocale.ENGLISH));
+        ms.setFormatByArgumentName("number", decimalFormat);
+        final SimpleDateFormat dateFormat = new SimpleDateFormat("'year:'yy 'month:'MM 'day:'dd");
+        dateFormat.setTimeZone(TimeZone.getTimeZone("Etc/GMT"));
+        ms.setFormatByArgumentName("date", dateFormat);
+        Map map = new HashMap();
+        map.put("number", new Integer(1234));
+        map.put("date", new Date(0,0,0));
+        String result = ms.format(map);
+        assertEquals("setFormatByArgumentName", "1234.000 year:99 month:12 day:31", result);
+        Set formatNames = ms.getFormatArgumentNames();
+        assertEquals("Format Names match", formatNames, map.keySet());
+        assertEquals("Decimal", decimalFormat, ms.getFormatByArgumentName("number"));
+        assertEquals("Date", dateFormat, ms.getFormatByArgumentName("date"));
+    }
 
 //#if defined(FOUNDATION10) || defined(J2SE13)
 //#else
@@ -1347,8 +1387,8 @@ public class TestMessageFormat extends com.ibm.icu.dev.test.TestFmwk {
         };
         
         String[] expectedStrings = {
-            "The 2nd folder 'tmp' contains 15 file(s), created at 9:58:08 AM on Jul 18, 2007.",
-            "The 2nd folder 'tmp' contains 15 file(s), created at 9:58:08 AM on Jul 18, 2007.",
+            "The 2\u207f\u1d48 folder 'tmp' contains 15 file(s), created at 9:58:08 AM on Jul 18, 2007.",
+            "The 2\u207f\u1d48 folder 'tmp' contains 15 file(s), created at 9:58:08 AM on Jul 18, 2007.",
             "The folder contains 34 files."
         };
 
