@@ -1,7 +1,7 @@
-//##header J2SE15
+//##header
 /*
  *******************************************************************************
- * Copyright (C) 1996-2007, International Business Machines Corporation and    *
+ * Copyright (C) 1996-2009, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  *
@@ -9,6 +9,7 @@
 
 package com.ibm.icu.dev.test.serializable;
 
+import java.text.AttributedCharacterIterator;
 import java.util.Date;
 import java.util.HashMap;
 import java.util.Locale;
@@ -18,7 +19,10 @@ import com.ibm.icu.text.ChineseDateFormat;
 import com.ibm.icu.text.ChineseDateFormatSymbols;
 import com.ibm.icu.text.DateFormat;
 import com.ibm.icu.text.DateFormatSymbols;
+import com.ibm.icu.text.DateIntervalFormat;
+import com.ibm.icu.text.DateIntervalInfo;
 import com.ibm.icu.text.DecimalFormat;
+import com.ibm.icu.text.CurrencyPluralInfo;
 import com.ibm.icu.text.DecimalFormatSymbols;
 import com.ibm.icu.text.DurationFormat;
 import com.ibm.icu.text.MessageFormat;
@@ -27,8 +31,12 @@ import com.ibm.icu.text.PluralFormat;
 import com.ibm.icu.text.PluralRules;
 import com.ibm.icu.text.RuleBasedNumberFormat;
 import com.ibm.icu.text.SimpleDateFormat;
+import com.ibm.icu.text.TimeUnitFormat;
 import com.ibm.icu.util.Calendar;
+import com.ibm.icu.util.DateInterval;
 import com.ibm.icu.util.GregorianCalendar;
+import com.ibm.icu.util.TimeUnit;
+import com.ibm.icu.util.TimeUnitAmount;
 import com.ibm.icu.util.ULocale;
 
 /**
@@ -1093,7 +1101,21 @@ public class FormatTests
                 
                 formats[i] = getCannedDecimalFormat("#,##0.###", uloc);
             }
-            
+//#if defined(FOUNDATION10) || defined(J2SE13)
+//#else
+            if (formats[0] != null) {
+                // Ticket#6449
+                // Once formatToCharacterIterator is called, NumberFormat.Field
+                // instances are created and stored in the private List field.
+                // NumberForamt.Field is not a serializable, so serializing such
+                // instances end up NotSerializableException.  This problem was
+                // reproduced since formatToCharacterIterator was introduced,
+                // up to ICU 4.0.
+
+                AttributedCharacterIterator aci = formats[0].formatToCharacterIterator(new Double(12.345D));
+                if (aci == null) {} // NOP - for resolving 'Unused local variable' warning.
+            }
+//#endif
             return formats;
         }
     }
@@ -1670,6 +1692,24 @@ public class FormatTests
         }
     }
     
+    public static class CurrencyPluralInfoHandler implements SerializableTest.Handler
+    {
+        public Object[] getTestObjects()
+        {
+            CurrencyPluralInfo currencyPluralInfo[] = {
+                new CurrencyPluralInfo()
+            };
+            currencyPluralInfo[0].setPluralRules("one: n is 1; few: n in 2..4");
+            currencyPluralInfo[0].setCurrencyPluralPattern("few", "few currency");
+            return currencyPluralInfo;
+        }
+        
+        public boolean hasSameBehavior(Object a, Object b)
+        {
+            return a.equals(b);
+        }
+    }
+    
     public static class MessageFormatHandler implements SerializableTest.Handler
     {
         public Object[] getTestObjects()
@@ -1763,6 +1803,27 @@ public class FormatTests
             String sfa = dfa.format(fixedDate);
             String sfb = dfb.format(fixedDate);
 
+            if (!sfa.equals(sfb)) {
+                // TODO
+                // In ICU3.8, localized GMT format pattern was added in
+                // DateFormatSymbols, which has no public setter.
+                // The difference of locale data for localized GMT format
+                // will produce different format result.  This is a temporary
+                // workaround for the issue.
+                DateFormatSymbols dfsa = ((SimpleDateFormat)dfa).getDateFormatSymbols();
+                DateFormatSymbols tmp = (DateFormatSymbols)((SimpleDateFormat)dfb).getDateFormatSymbols().clone();
+
+                tmp.setMonths(dfsa.getMonths());
+                tmp.setShortMonths(dfsa.getShortMonths());
+                tmp.setWeekdays(dfsa.getWeekdays());
+                tmp.setShortWeekdays(dfsa.getShortWeekdays());
+                tmp.setAmPmStrings(dfsa.getAmPmStrings());
+
+                ((SimpleDateFormat)dfa).setDateFormatSymbols(tmp);
+
+                sfa = dfa.format(fixedDate);
+            }
+
             return sfa.equals(sfb);
         }
         
@@ -1859,7 +1920,67 @@ public class FormatTests
             return dateFormats;
         }
     }
-    
+
+    public static class DateIntervalFormatHandler implements SerializableTest.Handler
+    {
+        public Object[] getTestObjects()
+        {
+            DateIntervalFormat dateIntervalFormats[] = {
+                DateIntervalFormat.getInstance("yMMMMEEEEd")
+            };
+            return dateIntervalFormats;
+        }
+
+        public boolean hasSameBehavior(Object a, Object b)
+        {
+            DateIntervalFormat dfa = (DateIntervalFormat) a;
+            DateIntervalFormat dfb = (DateIntervalFormat) b;
+            DateInterval dateInterval = new DateInterval(1, System.currentTimeMillis());
+            String sfa = dfa.format(dateInterval);
+            String sfb = dfb.format(dateInterval);
+
+            return sfa.equals(sfb);
+        }
+    }
+
+
+    public static class DateIntervalInfoHandler implements SerializableTest.Handler
+    {
+        public Object[] getTestObjects()
+        {
+            DateIntervalInfo dateIntervalInfo[] = {
+                new DateIntervalInfo()
+            };
+            dateIntervalInfo[0].setIntervalPattern("yMd", Calendar.YEAR, "yy/MM/dd - yy/MM/dd");
+            dateIntervalInfo[0].setIntervalPattern("yMd", Calendar.MONTH, "yy/MM - MM/dd");
+            return dateIntervalInfo;
+        }
+
+        public boolean hasSameBehavior(Object a, Object b)
+        {
+            return a.equals(b);
+        }
+    }
+
+
+    public static class PatternInfoHandler implements SerializableTest.Handler
+    {
+        public Object[] getTestObjects()
+        {
+            DateIntervalInfo.PatternInfo patternInfo[] = {
+                new DateIntervalInfo.PatternInfo("yyyy MMM dd - ", 
+                                                 "dd",
+                                                 false)
+            };
+            return patternInfo;
+        }
+
+        public boolean hasSameBehavior(Object a, Object b)
+        {
+            return a.equals(b);
+        }
+    }
+
     public static class ChineseDateFormatHandler extends DateFormatHandler
     {
         String patterns[] = {
@@ -1968,7 +2089,7 @@ public class FormatTests
             DateNumberFormat[] dnfmts = new DateNumberFormat[locales.length];
             for (int i = 0; i < locales.length; i++) {
                 ULocale uloc = ULocale.forLocale(locales[i]);
-                dnfmts[i] = new DateNumberFormat(uloc);
+                dnfmts[i] = new DateNumberFormat(uloc,'0');
             }
             return dnfmts;
         }
@@ -1978,48 +2099,76 @@ public class FormatTests
         }
     }
 
-  public static class PluralFormatHandler implements SerializableTest.Handler {
-    public Object[] getTestObjects() {
-      Locale[] locales = { Locale.US }; // main test is in plural rules handler
-      PluralFormat[] plfmts = new PluralFormat[locales.length];
-      for (int i = 0; i < locales.length; i++) {
-        ULocale uloc = ULocale.forLocale(locales[i]);
-        try {
-          plfmts[i] = new PluralFormat(uloc, "one{1 foo} other{# foo}");
-        } catch (Exception e) {
-          e.printStackTrace();
+    public static class PluralFormatHandler implements SerializableTest.Handler {
+        public Object[] getTestObjects() {
+            Locale[] locales = { Locale.US }; // main test is in plural rules handler
+            PluralFormat[] plfmts = new PluralFormat[locales.length];
+            for (int i = 0; i < locales.length; i++) {
+                ULocale uloc = ULocale.forLocale(locales[i]);
+                try {
+                    plfmts[i] = new PluralFormat(uloc, "one{1 foo} other{# foo}");
+                } catch (Exception e) {
+                    e.printStackTrace();
+                }
+            }
+            return plfmts;
+        }
+        public boolean hasSameBehavior(Object a, Object b) {
+            PluralFormat pfa = (PluralFormat)a;
+            PluralFormat pfb = (PluralFormat)b;
+            boolean isSame = true;
+            for (int i = 0; i < 10; i++) {
+                String texta = pfa.format(i);
+                String textb = pfb.format(i);
+                if (!texta.equals(textb)) {
+                    isSame = false;
+                    break;
+                }
+            }
+            return isSame;
         }
-      }
-      return plfmts;
     }
-    public boolean hasSameBehavior(Object a, Object b) {
-      return a.equals(b);
-    }
-  }
 
-  public static class PluralRulesHandler implements SerializableTest.Handler {
-    public Object[] getTestObjects() {
-      
-      String[] localeNames = {"ja","da","fr","lv","ga","ro","lt","hr","cs","pl","sl"};
-      PluralRules[] plrulz = new PluralRules[localeNames.length];
-      for (int i = 0; i < localeNames.length; i++) {
-        ULocale uloc = ULocale.createCanonical(localeNames[i]);
-        try {
-          plrulz[i] = PluralRules.forLocale(uloc);
-        } catch (Exception e) {
-          e.printStackTrace();
+    public static class PluralRulesHandler implements SerializableTest.Handler {
+        public Object[] getTestObjects() {
+            String[] localeNames = {"ja","da","fr","lv","ga","ro","lt","hr","cs","pl","sl"};
+            PluralRules[] plrulz = new PluralRules[localeNames.length];
+            for (int i = 0; i < localeNames.length; i++) {
+                ULocale uloc = ULocale.createCanonical(localeNames[i]);
+                try {
+                    plrulz[i] = PluralRules.forLocale(uloc);
+                } catch (Exception e) {
+                    e.printStackTrace();
+                }
+            }
+            return plrulz;
+        }
+        public boolean hasSameBehavior(Object a, Object b) {
+            return a.equals(b);
         }
-      }
-      return plrulz;
     }
-    public boolean hasSameBehavior(Object a, Object b) {
-      return a.equals(b);
-    }
-  }
     
+    public static class TimeUnitFormatHandler implements SerializableTest.Handler {
+        // TODO - more test coverage!
+        public Object[] getTestObjects() {
+            return new Object[] { new TimeUnitFormat().setLocale(ULocale.ENGLISH) };
+        }
+        public boolean hasSameBehavior(Object a, Object b) {
+            TimeUnitFormat tufa = (TimeUnitFormat)a;
+            TimeUnitFormat tufb = (TimeUnitFormat)b;
+
+            TimeUnitAmount amount = new TimeUnitAmount(3, TimeUnit.HOUR);
+            String resa = tufa.format(amount);
+            String resb = tufb.format(amount);
+
+            return resa.equals(resb);
+        }
+    }
+
     public static void main(String[] args)
     {
         // nothing needed...
     }
+
 }
 //eof
