@@ -1,7 +1,7 @@
-//##header J2SE15
+//##header
 /*
  *******************************************************************************
- * Copyright (C) 1996-2008, International Business Machines Corporation and    *
+ * Copyright (C) 1996-2009, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  *
@@ -545,7 +545,7 @@ public class SerializableTest extends TestFmwk.TestGroup
             TimeZone zone_a = (TimeZone) a;
             TimeZone zone_b = (TimeZone) b;
 
-            if (!(zone_a.getID().equals(zone_b.getID()))) {
+            if (!(TimeZone.getCanonicalID(zone_a.getID()).equals(TimeZone.getCanonicalID(zone_b.getID())))) {
                 return false;
             }
 
@@ -656,6 +656,7 @@ public class SerializableTest extends TestFmwk.TestGroup
         map.put("com.ibm.icu.text.NumberFormat", new FormatTests.NumberFormatHandler());
         map.put("com.ibm.icu.text.DecimalFormat", new FormatTests.DecimalFormatHandler());
         map.put("com.ibm.icu.text.RuleBasedNumberFormat", new FormatTests.RuleBasedNumberFormatHandler());
+        map.put("com.ibm.icu.text.CurrencyPluralInfo", new FormatTests.CurrencyPluralInfoHandler());
         map.put("com.ibm.icu.text.DecimalFormatSymbols", new FormatTests.DecimalFormatSymbolsHandler());
         map.put("com.ibm.icu.text.MessageFormat", new FormatTests.MessageFormatHandler());
         map.put("com.ibm.icu.text.DateFormat", new FormatTests.DateFormatHandler());
@@ -698,6 +699,9 @@ public class SerializableTest extends TestFmwk.TestGroup
 //#endif
         map.put("com.ibm.icu.impl.duration.BasicDurationFormat", new FormatTests.BasicDurationFormatHandler());
         map.put("com.ibm.icu.impl.RelativeDateFormat", new FormatTests.RelativeDateFormatHandler());
+        map.put("com.ibm.icu.util.IllformedLocaleException", new ExceptionTests.IllformedLocaleExceptionHandler());
+        map.put("com.ibm.icu.impl.locale.LocaleSyntaxException", new ExceptionTests.LocaleSyntaxExceptionHandler());
+        map.put("com.ibm.icu.impl.IllegalIcuArgumentException", new ExceptionTests.IllegalIcuArgumentExceptionHandler());
     }
     
     public SerializableTest()
