@@ -1,6 +1,6 @@
 /*
  *******************************************************************************
- * Copyright (C) 1996-2007, International Business Machines Corporation and    *
+ * Copyright (C) 1996-2009, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
@@ -73,8 +73,14 @@ public class UnicodeSetTest extends TestFmwk {
   }
 
   public void TestPropertyAccess() {
+    int count = 0; 
     // test to see that all of the names work
     for (int propNum = UProperty.BINARY_START; propNum < UProperty.INT_LIMIT; ++propNum) {
+      count++;
+      //Skipping tests in the non-exhaustive mode to shorten the test time ticket#6475
+      if(getInclusion()<=5 && count%5!=0){
+          continue;
+      }
       if (propNum >= UProperty.BINARY_LIMIT && propNum < UProperty.INT_START) { // skip the gap
         propNum = UProperty.INT_START;
       }
@@ -144,7 +150,7 @@ public class UnicodeSetTest extends TestFmwk {
                 + "Differing values: " + collectedErrors.toPattern(true));
           }
         }
-      }
+      } 
     }
   }
   
@@ -200,19 +206,26 @@ public class UnicodeSetTest extends TestFmwk {
     // Cover applyPattern, applyPropertyAlias
     s.clear();
     s.applyPattern("[ab ]", true);
-    expectToPattern(s, "[ab]", new String[] {"a", NOT, "ab"});
+    expectToPattern(s, "[ab]", new String[] {"a", NOT, "ab", " "});
     s.clear();
     s.applyPattern("[ab ]", false);
-    expectToPattern(s, "[\\\u0020ab]", new String[] {"a", "\u0020", NOT, "ab"});
+    expectToPattern(s, "[\\ ab]", new String[] {"a", "\u0020", NOT, "ab"});
     
     s.clear();
     s.applyPropertyAlias("nv", "0.5");
-    expectToPattern(s, "[\\u00BD\\u0F2A\\u2CFD\\U00010141\\U00010175\\U00010176]", null);
-    // Unicode 4.1 adds \u2CFD\U00010141\U00010175\U00010176 with numeric value 1/2
+    expectToPattern(s, "[\\u00BD\\u0D74\\u0F2A\\u2CFD\\U00010141\\U00010175\\U00010176]", null);
+    // Unicode 5.1 adds Malayalam 1/2 (\u0D74)
     
     s.clear();
     s.applyPropertyAlias("gc", "Lu");
     // TODO expectToPattern(s, what?)
+
+    // RemoveAllStrings()
+    s.clear();
+    s.applyPattern("[a-z{abc}{def}]");
+    expectToPattern(s, "[a-z{abc}{def}]", null);
+    s.removeAllStrings();
+    expectToPattern(s, "[a-z]", null);
   }
   
   static String[] OTHER_TOPATTERN_TESTS = {
@@ -1155,7 +1168,7 @@ public class UnicodeSetTest extends TestFmwk {
     if (uset.containsNone(string) != shouldContainNone) {
       errln("containsNone " +  string + " should be " + shouldContainNone);
     } else {
-      logln("containsNone " +  string + " = " + shouldContainNone);	    	
+      logln("containsNone " +  string + " = " + shouldContainNone);
     }
   }
   
@@ -1252,7 +1265,7 @@ public class UnicodeSetTest extends TestFmwk {
         // selector, input, output
         CASE,
         "[aq\u00DF{Bc}{bC}{Fi}]",
-        "[aAqQ\u00DF\uFB01{ss}{bc}{fi}]",
+        "[aAqQ\u00DF\u1E9E\uFB01{ss}{bc}{fi}]", // U+1E9E LATIN CAPITAL LETTER SHARP S is new in Unicode 5.1
         
         CASE,
         "[\u01F1]", // 'DZ'
@@ -1431,6 +1444,15 @@ public class UnicodeSetTest extends TestFmwk {
     expectEqual("POSIX print", "[:print:]", "[[:graph:][:blank:]-[\\p{Control}]]");
   }
   
+  public void TestHangulSyllable() {
+      final UnicodeSet lvt = new UnicodeSet("[:Hangul_Syllable_Type=LVT_Syllable:]");
+      assertNotEquals("LVT count", new UnicodeSet(), lvt);
+      logln(lvt + ": " + lvt.size());
+      final UnicodeSet lv = new UnicodeSet("[:Hangul_Syllable_Type=LV_Syllable:]");
+      assertNotEquals("LV count", new UnicodeSet(), lv);
+      logln(lv + ": " + lv.size());
+  }
+  
   /**
    * Test that frozen classes disallow changes. For 4217
    */
