@@ -1,6 +1,6 @@
  /*
   *******************************************************************************
-  * Copyright (C) 2005-2007, International Business Machines Corporation and         *
+  * Copyright (C) 2005-2009, International Business Machines Corporation and         *
   * others. All Rights Reserved.                                                *
   *******************************************************************************
   */
@@ -9,6 +9,7 @@ package com.ibm.icu.impl;
 import java.util.Arrays;
 import java.util.Date;
 
+import com.ibm.icu.util.AnnualTimeZoneRule;
 import com.ibm.icu.util.BasicTimeZone;
 import com.ibm.icu.util.Calendar;
 import com.ibm.icu.util.DateTimeRule;
@@ -19,7 +20,6 @@ import com.ibm.icu.util.TimeArrayTimeZoneRule;
 import com.ibm.icu.util.TimeZone;
 import com.ibm.icu.util.TimeZoneRule;
 import com.ibm.icu.util.TimeZoneTransition;
-import com.ibm.icu.util.ULocale;
 import com.ibm.icu.util.UResourceBundle;
 
 /**
@@ -173,21 +173,76 @@ public class OlsonTimeZone extends BasicTimeZone {
         if (getRawOffset() == offsetMillis) {
             return;
         }
-        GregorianCalendar cal = new GregorianCalendar(ULocale.ROOT);
-        cal.setTimeZone(this);
-        int tmpFinalYear = cal.get(Calendar.YEAR) - 1;
+        long current = System.currentTimeMillis();
+
+        if (current < finalMillis) {
+            SimpleTimeZone stz = new SimpleTimeZone(offsetMillis, getID());
+
+            boolean bDst = useDaylightTime();
+            if (bDst) {
+                TimeZoneRule[] currentRules = getSimpleTimeZoneRulesNear(current);
+                if (currentRules.length != 3) {
+                    // DST was observed at the beginning of this year, so useDaylightTime
+                    // returned true.  getSimpleTimeZoneRulesNear requires at least one
+                    // future transition for making a pair of rules.  This implementation
+                    // rolls back the time before the latest offset transition.
+                    TimeZoneTransition tzt = getPreviousTransition(current, false);
+                    if (tzt != null) {
+                        currentRules = getSimpleTimeZoneRulesNear(tzt.getTime() - 1);
+                    }
+                }
+                if (currentRules.length == 3
+                        && (currentRules[1] instanceof AnnualTimeZoneRule)
+                        && (currentRules[2] instanceof AnnualTimeZoneRule)) {
+                    // A pair of AnnualTimeZoneRule
+                    AnnualTimeZoneRule r1 = (AnnualTimeZoneRule)currentRules[1];
+                    AnnualTimeZoneRule r2 = (AnnualTimeZoneRule)currentRules[2];
+                    DateTimeRule start, end;
+                    int offset1 = r1.getRawOffset() + r1.getDSTSavings();
+                    int offset2 = r2.getRawOffset() + r2.getDSTSavings();
+                    int sav;
+                    if (offset1 > offset2) {
+                        start = r1.getRule();
+                        end = r2.getRule();
+                        sav = offset1 - offset2;
+                    } else {
+                        start = r2.getRule();
+                        end = r1.getRule();
+                        sav = offset2 - offset1;
+                    }
+                    // getSimpleTimeZoneRulesNear always return rules using DOW / WALL_TIME
+                    stz.setStartRule(start.getRuleMonth(), start.getRuleWeekInMonth(), start.getRuleDayOfWeek(),
+                                            start.getRuleMillisInDay());
+                    stz.setEndRule(end.getRuleMonth(), end.getRuleWeekInMonth(), end.getRuleDayOfWeek(),
+                                            end.getRuleMillisInDay());
+                    // set DST saving amount and start year
+                    stz.setDSTSavings(sav);
+                } else {
+                    // This could only happen if last rule is DST
+                    // and the rule used forever.  For example, Asia/Dhaka
+                    // in tzdata2009i stays in DST forever.
+
+                    // Hack - set DST starting at midnight on Jan 1st,
+                    // ending 23:59:59.999 on Dec 31st
+                    stz.setStartRule(0, 1, 0);
+                    stz.setEndRule(11, 31, Grego.MILLIS_PER_DAY - 1);
+                }
+            }
+
+            int[] fields = Grego.timeToFields(current, null);
+            finalYear = fields[0] - 1; // finalYear is (year of finalMillis) - 1
+            finalMillis = Grego.fieldsToDay(fields[0], 0, 1);
+
+            if (bDst) {
+                // we probably do not need to set start year of final rule
+                // to finalzone itself, but we always do this for now.
+                stz.setStartYear(finalYear);
+            }
+
+            finalZone = stz;
 
-        // Apply the raw offset starting current year and beyond
-        if (finalYear > tmpFinalYear) {
-            finalYear = tmpFinalYear;
-            finalMillis = Grego.fieldsToDay(tmpFinalYear, 0, 1) * Grego.MILLIS_PER_DAY;
-        }
-        if (finalZone == null) {
-            // Create SimpleTimeZone instance to store the offset
-            finalZone = new SimpleTimeZone(offsetMillis, getID());
         } else {
             finalZone.setRawOffset(offsetMillis);
-            finalZone.setStartYear(finalYear);
         }
 
         transitionRulesInitialized = false;
@@ -269,8 +324,8 @@ public class OlsonTimeZone extends BasicTimeZone {
             if (transitionTimes[i] >= limit) {
                 break;
             }
-            if (transitionTimes[i] >= start &&
-                dstOffset(typeData[i]) != 0) {
+            if ((transitionTimes[i] >= start && dstOffset(typeData[i]) != 0)
+                    || (transitionTimes[i] > start && i > 0 && dstOffset(typeData[i - 1]) != 0)) {
                 return true;
             }
         }
@@ -490,7 +545,7 @@ public class OlsonTimeZone extends BasicTimeZone {
     private void getHistoricalOffset(long date, boolean local,
             int NonExistingTimeOpt, int DuplicatedTimeOpt, int[] offsets) {
         if (transitionCount != 0) {
-            long sec = myFloorDivide(date, Grego.MILLIS_PER_SECOND);
+            long sec = Grego.floorDivide(date, Grego.MILLIS_PER_SECOND);
             // Linear search from the end is the fastest approach, since
             // most lookups will happen at/near the end.
             int i = 0;
@@ -680,25 +735,6 @@ public class OlsonTimeZone extends BasicTimeZone {
         return r;
     }
 
-    /**
-     * Divide two long integers, returning the floor of the quotient.
-     * <p>
-     * Unlike the built-in division, this is mathematically well-behaved.
-     * E.g., <code>-1/4</code> => 0
-     * but <code>floorDivide(-1,4)</code> => -1.
-     * @param numerator the numerator
-     * @param denominator a divisor which must be > 0
-     * @return the floor of the quotient.
-     * @stable ICU 2.0
-     */
-    private static final long myFloorDivide(long numerator, long denominator) {
-        // We do this computation in order to handle
-        // a numerator of Long.MIN_VALUE correctly
-        return (numerator >= 0) ?
-            numerator / denominator :
-            ((numerator + 1) / denominator) - 1;
-    }
-
     public boolean equals(Object obj){
         if (!super.equals(obj)) return false; // super does class check
         
