@@ -1,13 +1,14 @@
 /*
-**********************************************************************
-* Copyright (c) 2002-2006, International Business Machines
-* Corporation and others.  All Rights Reserved.
-**********************************************************************
-* Author: Alan Liu
-* Created: November 5 2002
-* Since: ICU 2.4
-**********************************************************************
-*/
+ **********************************************************************
+ * Copyright (c) 2002-2009, International Business Machines
+ * Corporation and others.  All Rights Reserved.
+ **********************************************************************
+ * Author: Alan Liu
+ * Created: November 5 2002
+ * Since: ICU 2.4
+ **********************************************************************
+ */
+
 package com.ibm.icu.impl;
 
 import java.io.*;
@@ -299,7 +300,7 @@ public final class UPropertyAliases implements ICUBinary.Authenticate {
 
         public short getShort(int enumProbe) {
             if (enumProbe < enumStart || enumProbe >= enumLimit) {
-                throw new IllegalArgumentException("Invalid enum. enumStart = " +enumStart +
+                throw new IllegalIcuArgumentException("Invalid enum. enumStart = " +enumStart +
                                                    " enumLimit = " + enumLimit +
                                                    " enumProbe = " + enumProbe );
             }
@@ -331,7 +332,7 @@ public final class UPropertyAliases implements ICUBinary.Authenticate {
                 if (enumArray[i] > enumProbe) break;
                 return offsetArray[i];
             }
-            throw new IllegalArgumentException("Invalid enum");
+            throw new IllegalIcuArgumentException("Invalid enum");
         }
 
         NonContiguousEnumToShort(ICUBinaryStream s) throws IOException  {
@@ -363,7 +364,7 @@ public final class UPropertyAliases implements ICUBinary.Authenticate {
                 if (c < 0) break;
                 return enumArray[i];
             }
-            throw new IllegalArgumentException("Invalid name: " + nameProbe);
+            throw new IllegalIcuArgumentException("Invalid name: " + nameProbe);
         }
 
         NameToEnum(Builder b) throws IOException {
@@ -443,11 +444,11 @@ public final class UPropertyAliases implements ICUBinary.Authenticate {
      */
     private String chooseNameInGroup(short nameGroupIndex, int nameChoice) {
         if (nameChoice < 0) {
-            throw new IllegalArgumentException("Invalid name choice");
+            throw new IllegalIcuArgumentException("Invalid name choice");
         }
         while (nameChoice-- > 0) {
             if (nameGroupPool[nameGroupIndex++] < 0) {
-                throw new IllegalArgumentException("Invalid name choice");
+                throw new IllegalIcuArgumentException("Invalid name choice");
             }
         }
         short a = nameGroupPool[nameGroupIndex];
@@ -594,13 +595,11 @@ public final class UPropertyAliases implements ICUBinary.Authenticate {
          * array of stringPool[] indices.  MODIFIES THE ARRAY IN
          * PLACE.
          */
-        ///CLOVER:OFF
-        private void stringOffsetToIndex(short array[]) {
+/*        private void stringOffsetToIndex(short array[]) {
             for (int i=0; i<array.length; ++i) {
                 array[i] = stringOffsetToIndex(array[i]);
             }
-        }
-        ///CLOVER:ON
+        }*/
 
         /**
          * Convert an offset into the value map into a valueMap[]
