@@ -81,16 +81,16 @@ public class ResourceReader {
         _reset();
     }
 
-    /**
-     * Construct a reader object for the input stream associated with
-     * the given resource name.
-     * @param is the input stream of the resource
-     * @param resourceName the name of the resource
-     */
-     public ResourceReader(InputStream is, String resourceName) {
-         this.root = null;
+         /**
+          * Construct a reader object for the input stream associated with
+          * the given resource name.
+          * @param is the input stream of the resource
+          * @param resourceName the name of the resource
+          */
+          public ResourceReader(InputStream is, String resourceName, String encoding) {
+                   this.root = null;
          this.resourceName = resourceName;
-         this.encoding = null;
+         this.encoding = encoding;
 
          this.lineNo = -1;
          try {
@@ -105,6 +105,16 @@ public class ResourceReader {
          }
      }
 
+          /**
+           * Construct a reader object for the input stream associated with
+           * the given resource name.
+           * @param is the input stream of the resource
+           * @param resourceName the name of the resource
+           */
+          public ResourceReader(InputStream is, String resourceName) {
+              this(is, resourceName, null);
+          }
+
     /**
      * Construct a reader object for the text file of the given name
      * in the given class's package, using the default encoding.
