@@ -1,6 +1,6 @@
 /*
  *******************************************************************************
- * Copyright (C) 1996-2008, International Business Machines Corporation and    *
+ * Copyright (C) 1996-2010, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
@@ -15,6 +15,7 @@ import java.util.MissingResourceException;
 import java.util.ResourceBundle;
 
 import com.ibm.icu.impl.CalendarData;
+import com.ibm.icu.impl.CalendarUtil;
 import com.ibm.icu.impl.ICUCache;
 import com.ibm.icu.impl.ICUResourceBundle;
 import com.ibm.icu.impl.SimpleCache;
@@ -164,7 +165,7 @@ public class DateFormatSymbols implements Serializable, Cloneable {
      */
     public DateFormatSymbols(ULocale locale)
     {
-        initializeData(locale, getCalendarType(locale));
+        initializeData(locale, CalendarUtil.getCalendarType(locale));
     }
 
     /**
@@ -208,8 +209,7 @@ public class DateFormatSymbols implements Serializable, Cloneable {
      * 
      * @param locale the locale.
      * @return A DateFormatSymbols instance.
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public static DateFormatSymbols getInstance(ULocale locale) {
         return new DateFormatSymbols(locale);
@@ -241,11 +241,11 @@ public class DateFormatSymbols implements Serializable, Cloneable {
      * or its equivalent implementation for now.
      * 
      * @return An array of <code>ULocale</code>s for which localized <code>DateFormatSymbols</code> instances are available.
-     * @draft ICU 3.8
+     * @draft ICU 3.8 (retain)
      * @provisional This API might change or be removed in a future release.
      */
     public static ULocale[] getAvailableULocales() {
-        return ICUResourceBundle.getAvailableULocales(ICUResourceBundle.ICU_BASE_NAME);        
+        return ICUResourceBundle.getAvailableULocales(ICUResourceBundle.ICU_BASE_NAME);
     }
     
     /**
@@ -1534,7 +1534,7 @@ public class DateFormatSymbols implements Serializable, Cloneable {
      */
     public DateFormatSymbols(ResourceBundle bundle, ULocale locale) {
         initializeData(locale, 
-            new CalendarData((ICUResourceBundle)bundle, getCalendarType(locale)));
+            new CalendarData((ICUResourceBundle)bundle, CalendarUtil.getCalendarType(locale)));
     }
 
     /**
@@ -1607,17 +1607,6 @@ public class DateFormatSymbols implements Serializable, Cloneable {
         return null;
     }
 
-    // Return the calendar type string for the given locale
-    private static String getCalendarType(ULocale locale) {
-        String calType = locale.getKeywordValue("calendar");
-        if (calType == null) {
-            locale = ICUResourceBundle.getFunctionalEquivalent(
-                ICUResourceBundle.ICU_BASE_NAME, "calendar", "calendar", locale, null, false);
-            calType = locale.getKeywordValue("calendar");
-        }
-        return calType;
-    }
-
     // -------- BEGIN ULocale boilerplate --------
 
     /**
