@@ -1,6 +1,6 @@
 /*
  *******************************************************************************
- * Copyright (C) 1996-2007, International Business Machines Corporation and    *
+ * Copyright (C) 1996-2009, International Business Machines Corporation and    *
  * others. All Rights Reserved.                                                *
  *******************************************************************************
  */
@@ -40,7 +40,7 @@ import java.util.MissingResourceException;
  * @stable ICU 2.0
  */
 
-final public class DecimalFormatSymbols implements Cloneable, Serializable {
+public class DecimalFormatSymbols implements Cloneable, Serializable {
 
     /**
      * Create a DecimalFormatSymbols object for the default locale.
@@ -109,8 +109,7 @@ final public class DecimalFormatSymbols implements Cloneable, Serializable {
      * 
      * @param locale the locale.
      * @return A DecimalFormatSymbols instance.
-     * @draft ICU 3.8
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.8
      */
     public static DecimalFormatSymbols getInstance(ULocale locale) {
         return new DecimalFormatSymbols(locale);
@@ -142,11 +141,11 @@ final public class DecimalFormatSymbols implements Cloneable, Serializable {
      * or its equivalent implementation for now.
      * 
      * @return An array of <code>ULocale</code>s for which localized <code>DecimalFormatSymbols</code> instances are available.
-     * @draft ICU 3.8
+     * @stable ICU 3.8 (retain)
      * @provisional This API might change or be removed in a future release.
      */
     public static ULocale[] getAvailableULocales() {
-        return ICUResourceBundle.getAvailableULocales(ICUResourceBundle.ICU_BASE_NAME);        
+        return ICUResourceBundle.getAvailableULocales(ICUResourceBundle.ICU_BASE_NAME);
     }
 
     
@@ -449,8 +448,7 @@ final public class DecimalFormatSymbols implements Cloneable, Serializable {
     /**
      * Return the monetary decimal separator.
      * @return the monetary decimal separator character
-     * @draft ICU 3.6
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.6
      */
     public char getMonetaryGroupingSeparator()
     {
@@ -477,8 +475,7 @@ final public class DecimalFormatSymbols implements Cloneable, Serializable {
     /**
      * Set the monetary decimal separator.
      * @param sep the monetary decimal separator character
-     * @draft ICU 3.6
-     * @provisional This API might change or be removed in a future release.
+     * @stable ICU 3.6
      */
     public void setMonetaryGroupingSeparator(char sep)
     {
@@ -573,7 +570,71 @@ final public class DecimalFormatSymbols implements Cloneable, Serializable {
     public void setPadEscape(char c) {
         padEscape = c;
     }
+    
+    public static final int CURRENCY_SPC_CURRENCY_MATCH = 0;
+    public static final int CURRENCT_SPC_SURROUNDING_MATCH = 1;
+    public static final int CURRENCY_SPC_INSERT = 2;
 
+    private static final String CURRENCY_SPACING = "currencySpacing";
+    private static final String BEFORE_CURRENCY = "beforeCurrency";
+    private static final String AFTER_CURRENCY = "afterCurrency";
+    private static final String[] CURRENCY_SPACING_KEYS = 
+        { "currencyMatch", "surroundingMatch" ,"insertBetween"};
+    private static final String DEFAULT_SPC_MATCH = "[:letter:]";
+    private static final String DEFAULT_SPC_INSERT = " ";
+    private static final String DEFAULT_SPC_SUR_MATCH = "[:digit:]";
+
+    private String[] currencySpcBeforeSym; // before currency symbol.
+    private String[] currencySpcAfterSym; // after currency symbol.
+    /**
+     * Get pattern string for 'CurrencySpacing' that can be applied to
+     * currency format.
+     * This API gets the CurrencySpacing data from ResourceBundle. The pattern can
+     * be null if there is no data from current locale and its parent locales.
+     *
+     * @param itemType : CURRENCY_SPC_CURRENCY_MATCH, CURRENCY_SPC_SURROUNDING_MATCH
+     *                   or CURRENCY_SPC_INSERT
+     * @param beforeCurrency : true if the pattern is for before currency symbol.
+     *                         false if the pattern is for after currency symbol.
+     * @return  pattern string for currencyMatch, surroundingMatch or spaceInsert.
+     *     Return null of there is no data for this locale and its parent locales.
+     */
+    public String getPatternForCurrencySpacing(int itemType, boolean beforeCurrency)  {
+        if (itemType < CURRENCY_SPC_CURRENCY_MATCH ||
+            itemType > CURRENCY_SPC_INSERT ) {
+            return null;  // invalid itemType.
+        }
+        if (beforeCurrency) {
+            return currencySpcBeforeSym[itemType];
+        } else {
+            return currencySpcAfterSym[itemType];
+        }
+    }
+
+    
+    /**
+     * Set pattern string for 'CurrencySpacing' that can be applied to
+     * CurrencyFormat.
+     * 
+     * @param itemType : CURRENCY_SPC_CURRENCY_MATCH, CURRENCY_SPC_SURROUNDING_MATCH
+     *                   or CURRENCY_SPC_INSERT
+     * @param beforeCurrency : true if the pattern is for before currency symbol.
+     *                         false if the pattern is for after currency symbol.
+     * @param  pattern string to override current setting.
+     */
+    public void setPatternForCurrencySpacing(int itemType, boolean beforeCurrency, String pattern) {
+
+        if (itemType < CURRENCY_SPC_CURRENCY_MATCH ||
+            itemType > CURRENCY_SPC_INSERT ) {
+            return;  // invalid itemType.
+        }
+        if (beforeCurrency) {
+            currencySpcBeforeSym[itemType] = pattern;
+        } else {
+            currencySpcAfterSym[itemType] = pattern;
+        }
+    }
+    
     /**
      * Returns the locale for which this object was constructed.
      * @return the locale for which this object was constructed
@@ -615,6 +676,15 @@ final public class DecimalFormatSymbols implements Cloneable, Serializable {
         if (obj == null) return false;
         if (this == obj) return true;
         DecimalFormatSymbols other = (DecimalFormatSymbols) obj;
+        for (int i = 0; i <= CURRENCY_SPC_INSERT; i++) {
+            if (!currencySpcBeforeSym[i].equals(other.currencySpcBeforeSym[i])) {
+                return false;
+            }
+            if (!currencySpcAfterSym[i].equals(other.currencySpcAfterSym[i])) {
+                return false;
+            }
+        }
+
         return (zeroDigit == other.zeroDigit &&
         groupingSeparator == other.groupingSeparator &&
         decimalSeparator == other.decimalSeparator &&
@@ -703,6 +773,13 @@ final public class DecimalFormatSymbols implements Cloneable, Serializable {
         padEscape = DecimalFormat.PATTERN_PAD_ESCAPE;
         sigDigit  = DecimalFormat.PATTERN_SIGNIFICANT_DIGIT;
         
+        // Attempt to set the zero digit based on the numbering system for the locale requested
+        //
+        NumberingSystem ns = NumberingSystem.getInstance(locale);
+        if ( ns != null && ns.getRadix() == 10 && !ns.isAlgorithmic()) {
+            zeroDigit = ns.getDescription().charAt(0);
+        }
+
         // Obtain currency data from the currency API.  This is strictly
         // for backward compatibility; we don't use DecimalFormatSymbols
         // for currency data anymore.
@@ -749,6 +826,20 @@ final public class DecimalFormatSymbols implements Cloneable, Serializable {
             /* else no currency keyword used. */
         }
         //monetarySeparator = numberElements[11].charAt(0);
+        
+        // Get Currency Spacing data. 
+        currencySpcBeforeSym = new String[CURRENCY_SPC_INSERT+1];
+        currencySpcAfterSym = new String[CURRENCY_SPC_INSERT+1];
+        ICUResourceBundle curSpcBundle = (ICUResourceBundle)r.get(CURRENCY_SPACING);
+        if (curSpcBundle != null) {
+            ICUResourceBundle beforeCurBundle = (ICUResourceBundle)curSpcBundle.get(BEFORE_CURRENCY);
+            ICUResourceBundle afterCurBundle = (ICUResourceBundle)curSpcBundle.get(AFTER_CURRENCY);
+            for (int i = CURRENCY_SPC_CURRENCY_MATCH; i <= CURRENCY_SPC_INSERT; i++) {
+                currencySpcBeforeSym[i] = beforeCurBundle.getStringWithFallback(CURRENCY_SPACING_KEYS[i]);
+                currencySpcAfterSym[i] = afterCurBundle.getStringWithFallback(CURRENCY_SPACING_KEYS[i]);
+            }
+        }
+        
     }
 
     /**
@@ -800,6 +891,21 @@ final public class DecimalFormatSymbols implements Cloneable, Serializable {
             // use the same one for groupingSeparator
             monetaryGroupingSeparator = groupingSeparator;
         }
+        if (serialVersionOnStream < 6) {
+            // Set null to CurrencySpacing related fields.
+            if (currencySpcBeforeSym == null) {
+                currencySpcBeforeSym = new String[CURRENCY_SPC_INSERT+1];
+            }
+            if (currencySpcAfterSym == null) {
+                currencySpcAfterSym = new String[CURRENCY_SPC_INSERT+1];
+            }
+            currencySpcBeforeSym[CURRENCY_SPC_CURRENCY_MATCH] = 
+                currencySpcAfterSym[CURRENCY_SPC_CURRENCY_MATCH] = DEFAULT_SPC_MATCH;
+            currencySpcBeforeSym[CURRENCT_SPC_SURROUNDING_MATCH] = 
+                currencySpcAfterSym[CURRENCT_SPC_SURROUNDING_MATCH] = DEFAULT_SPC_SUR_MATCH;
+            currencySpcBeforeSym[CURRENCY_SPC_INSERT] = 
+                currencySpcAfterSym[CURRENCY_SPC_INSERT] = DEFAULT_SPC_INSERT;
+        }
         serialVersionOnStream = currentSerialVersion;
 
     // recreate
@@ -983,7 +1089,8 @@ final public class DecimalFormatSymbols implements Cloneable, Serializable {
     // - 3 for ICU 2.2, which includes the locale field
     // - 4 for ICU 3.2, which includes the ULocale field
     // - 5 for ICU 3.6, which includes the monetaryGroupingSeparator field
-    private static final int currentSerialVersion = 5;
+    // - 6 for ICU 4.2, which includes the currencySpc* fields
+    private static final int currentSerialVersion = 6;
     
     /**
      * Describes the version of <code>DecimalFormatSymbols</code> present on the stream.
